////////////////////////////////////////////////////////////////////////////////
// 
// File:
//	test.cpp
// Package:
//	CGILib
// Author:
// 	Ignacio A. Macas <mack@las.es>
// Description:
//	How to use the CGILib: example.
//		Tested: GCC 2.7.2
//		Tested: MS Visual C++ 4.0
// Support:
// 	Please contact <mack@las.es> if you've any problem. But (please), first
//	you should read the documentation file "CGILib.html".
// Version:
// 	v1.0 Nov 4 1996 - First build.
// 
////////////////////////////////////////////////////////////////////////////////

# include "CGILib.h"
			// You must include the CGILib header.

const maxEnvVars = 19;
			// Total of environment variables in the array.
static char *envVars[maxEnvVars] =
			// Array with almost all the meaning environment
			// variables on CGI programming.
	{
	"CONTENT_LENGTH", "CONTENT_TYPE", "GATEWAY_INTERFACE","HTTP_ACCEPT",
	"HTTP_USER_AGENT", "HTTP_REFERER", "PATH_INFO", "PATH_TRANSLATED",
	"QUERY_STRING", "REMOTE_ADDR", "REMOTE_HOST", "REMOTE_IDENT",
	"REMOTE_USER", "REQUEST_METHOD", "SCRIPT_NAME", "SERVER_NAME",
	"SERVER_PORT", "SERVER_PROTOCOL","SERVER_SOFTWARE"
	};

int main()
{
CGIParser parser;
			// The only CGIParser object you must declare.

cout << "Content-type: text/html" << endl << endl;
			// Type of the data. Everything you write on the
			// standard output will be interpreted as HTML code
			// by your browser.
cout << "<HTML> <HEAD> <TITLE> Testing CGILib </TITLE> </HEAD>"
	"<BODY BGCOLOR=\"#DDDDDD\" TEXT=\"#000000\">"
	"<H1> Testing CGILib </H1> <HR>" << endl;
			// HTML header and title of the page.
try
	{
	parser.parse();
			// The parser examines the data sent by the server to
			// this program and stores it conveniently.

	cout << "<H2> Received data </H2>" << endl;

	cout << "Type : <I>" << parser.getType() << "</I> <BR>" << endl;
			// Type of the transaction: "ISINDEX", "GET" or "POST";
	cout << "Text : [<I>" << parser.getText() << "</I>] <BR>" << endl;
			// Traslated text of the transaction. Maybe for
			// debug your program.
	CGIData data = parser.getData();
			// We obtain a CGIData object which contains a map of
			// pairs name-value. "Name" is the identifier you used
			// in the HTML form and "Value" is the user data.
			// Eg. If your <FORM> contains a field like this:
			//	<INPUT TYPE="text" NAME="user" SIZE=20>
			// And the user types in it his name, "Mack", the pair
			// generated is "user=Mack". You can test the map using
			// the operator[] like this:
			//	if (data["user"] == "Mack")
			//		cout << "Hello, <B> Mack </B>";
			//      else
			//		cout << "How are you, <I>"
			//                   << data["user"] << "? </I>";
			// Renember that if a name is not defined, the
			// operator[] will return an empty string.
			// NOTE: Some implementations of the string::operator==
			// don't accept to check acainst a c-stype string.
			// Instead, you can use:
			//	if (!data["user"].compare("Mack"))
			//		...
			// Or:
			//	if (data["user"] == string("Mack"))
			//		...
			// You must renember that if the type is "ISNIDEX"
			// The CGIData map is Empty, you should use
			// getText() to access the user input.
		
	cout << "Data : <UL>" << endl;
	CGIData::iterator itr;
			// How to obtain all the pairs of the map.
	for ( itr = data.begin() ; itr != data.end() ; itr++)
		cout << "<LI> <I>" << (*itr).first << " = "
				<< (*itr).second << "</I> </LI>" << endl;
			// "first" is the "name" and "second" is the "value".
	cout << "</UL> <HR> <H2> Environment variables </H2> <UL>" << endl;
			// Lets show the values of the environment variables.
	for (int index = 0 ; index < maxEnvVars ; index++)
		cout << "<LI> " << envVars[index] << " = <I>"
			<< parser.getEnv(envVars[index]) << " </I></LI>" << endl;
	cout << "</UL>" << endl;
	}
	catch (CGIException e)
			// CGIparser member functions can throw instances of
			// CGIException if something unexpected happens.
			// You should catch that cases.
		{
		cout << "<BR> <BLINK> CGIException thrown: " << e.getText()
				<< "</BLINK>" << endl;
		}

cout << "<HR>&copy; Mack'96 [<A HREF=\"mailto:mack@las.es\">mack@las.es</A>] </BODY> </HTML>" << endl;
			// Bottom of the HTML page.
return 1;
}